#
# $Copyright: Copyright (c) 2009 Symantec Corporation.
# All rights reserved.
#
# THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF
# SYMANTEC CORPORATION.  USE, DISCLOSURE OR REPRODUCTION IS PROHIBITED
# WITHOUT THE PRIOR EXPRESS WRITTEN PERMISSION OF SYMANTEC CORPORATION.
#
# The Licensed Software and Documentation are deemed to be commercial
# computer software as defined in FAR 12.212 and subject to restricted
# rights as defined in FAR Section 52.227-19 "Commercial Computer
# Software - Restricted Rights" and DFARS 227.7202, "Rights in
# Commercial Computer Software or Commercial Computer Software
# Documentation", as applicable, and any successor regulations. Any use,
# modification, reproduction release, performance, display or disclosure
# of the Licensed Software and Documentation by the U.S. Government
# shall be solely in accordance with the terms of this Agreement.  $
#

package SRDFStarAgent;
#define hash
use ArrayRep;
@ISA = ("ArrayRep");

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %default);
use ag_i18n_inc;
use File::Basename;

my ($vcs_home, $vcs_log, $SYMSTAR, $SYMRDF);
my (@CMDS, $_DO, $_DOE);

$vcs_home=$VCSHOME;
$vcs_log=$VCSLOG;
$SYMSTAR = "/bin/symstar";
$SYMRDF = "/bin/symrdf";

@EXPORT = qw(
);

$VERSION = '1.00';

require 5.000;

%default = {
	localSite => '',
	firstSite => '',
	secondSite => '',
	currentWorkloadSite => '',
	currentSyncSite => '',
	currentAsyncSite => '',
	resource_type => 'SRDF',
};

sub new {
	my($proto, $resname, $symHome, $name, $syncSite, $asyncSite, $useMostRecentData, $keepDataOn, $keepDataFrom, $onlineOnState, $tripTakeover, $forceOnline, $haltOnOffline) = @_;
	my $class = ref($proto) || $proto;
	my $self = { %default };
	my (%stateinfo) = ();

	my @query;
	my $workloadSite;
	my $syncTargetSite;
	my $asyncTargetSite;
	my $mostRecentSite; 
	my $starEnabled;

	$self->{'resname'} = $resname;
	$self->{'home'} = $symHome;
	$self->{'name'} = $name;
	$self->{'syncSite'} = $syncSite;
	$self->{'asyncSite'} = $asyncSite;
	$self->{'useMostRecentData'} = $useMostRecentData;
	$self->{'keepDataOn'} = $keepDataOn;	
	$self->{'keepDataFrom'} = $keepDataFrom;
	$self->{'onlineOnState'} = $onlineOnState;
	$self->{'tripTakeover'} = $tripTakeover;
	$self->{'forceOnline'} = $forceOnline;
	$self->{'haltOnOffline'} = $haltOnOffline;

	$self->{'starEnabled'} = 0;	
	$self->{'actionTimeout'} = 86400;

	$self->{STATEINFO} = \%stateinfo;
	bless($self, $class);
	return $self;
}

sub DESTROY {}

sub get_star_info() {
	my ($self) = @_;
	my ($retVal, $cmd, $ret, $localSite, $firstSite, $secondSite, $starstate, $resname, $k);

	$retVal = 1;
	$ret = $self->get_query_output();
	if ($ret) {
		return $retVal;
	}
	$ret = $self->get_site_names();
	if ($ret) {
		return $retVal;
	}
	$localSite = $self->{'localSite'}; 
	$firstSite = $self->{'firstSite'};
	$secondSite = $self->{'secondSite'};
	$self->{STATEINFO}{$localSite} = "NA";
	$self->{STATEINFO}{$firstSite} = "NA";
	$self->{STATEINFO}{$secondSite} = "NA";
	$self->{STATEINFO}{"STAR"} = "Unprotected";

	$ret = $self->get_site_state("1st_Target_Site");
	if ($ret) {
		return $retVal;
	}
	$ret = $self->get_site_state("2nd_Target_Site");
	if ($ret) {
		return $retVal;
	}
	$ret = $self->get_most_recent_site();
	if ($ret) {
		return $retVal;
	}
	$ret = $self->get_star_state();
	if ($ret) {
		return $retVal;
	}

	foreach $k (keys (%{$self->{STATEINFO}})) {
		VCSAG_LOG_MSG("I", "State of $k is $self->{STATEINFO}{$k}\n", 11, $k, $self->{STATEINFO}{$k}, $k);
	}

	$starstate = $self->{STATEINFO}{"STAR"};
	$resname = $self->{'resname'};
	$cmd = "$vcs_home/bin/hares -modify $resname StarState $starstate";
	$ret = system($cmd); 
	if ($ret) {
		VCSAG_LOG_MSG("E", "Error executing $cmd\n", 80, $cmd);
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub get_query_output() {
	my ($self) = @_;
	my ($retVal, $cmd);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
	$cmd = $cmd . " -cg $self->{'name'} query -detail";

	VCSAG_LOG_MSG("N", "issuing command: $cmd", 101, $cmd);
	$self->{'query'} = `$cmd`;
	if (!$self->{'query'}) {
		VCSAG_LOG_MSG("E", "Command failed:\n $cmd \n", 62, $cmd);
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub get_site_names(){
	my ($self) = @_;
	my ($retVal, @output, @query, $q);

	$retVal = 1;
	@query = split('\n', $self->{'query'});
		
	foreach $q (@query) {
		if ($q =~ /Site Name/) {
			@output= split('\s+', $q);
			$self->{'localSite'} = $output[3];
		} elsif ($q =~ /Workload Site/) {
			@output= split('\s+', $q);
			$self->{'currentWorkloadSite'} = $output[3];
		} elsif ($q =~ /1st Target Site/) {
			@output= split('\s+', $q);
			$self->{'firstSite'} = $output[4];
		} elsif ($q =~ /2nd Target Site/) {
			@output= split('\s+', $q);
			$self->{'secondSite'} = $output[4];
		} elsif ($q =~ /Synchronous Target Site/) {
			@output= split('\s+', $q);
			$self->{'currentSyncSite'} = $output[5];	
		} elsif ($q =~ /Asynchronous Target Site/) {
			@output= split('\s+', $q);
			$self->{'currentAsyncSite'} = $output[5];
			last;	
		}
	}

	if (!$self->{'localSite'} || !$self->{'firstSite'} || !$self->{'secondSite'} || 
		!$self->{'currentWorkloadSite'} || !$self->{'currentSyncSite'} || !$self->{'currentAsyncSite'}) {
		VCSAG_LOG_MSG("E", "Error getting names of the sites\n", 12);
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub get_most_recent_site() {
	my ($self) = @_;
	my ($retVal, @query, @output, $q);
	
	$retVal = 1;
	@query = split('\n', $self->{'query'});

	foreach $q (@query) {
		if ($q =~ /Asynchronous Target Site Data most Current/) {
			@output = split('\s+', $q);
			if ($output[8] eq "Yes") {
				$self->{'mostRecentSite'} = $self->{'currentAsyncSite'};
			}
			elsif ($output[8] eq "No") {
				$self->{'mostRecentSite'} = $self->{'currentSyncSite'};
			} else {
				$self->{'mostRecentSite'} = $self->{'localSite'};
			}
			$retVal = 0;
			last;
		}
	}

	return $retVal;
} 

sub get_site_state() {
	my ($self, $site_name) = @_;
	my ($retVal, $q, @query, @site_state);

	$retVal = 1;
	@query = split('\n',$self->{'query'});

	foreach $q (@query) {
		if ($q =~ /$site_name/) {
			@site_state = split('\s+', $q);
			if ($site_name eq "1st_Target_Site") {
				$site_name = $self->{'firstSite'};
			}
			if ($site_name eq "2nd_Target_Site") {
				$site_name = $self->{'secondSite'};
			}
			$self->{STATEINFO}{$site_name} = $site_state[3];
			$retVal = 0;
			last;
		}
	}

	if ($retVal) {
                VCSAG_LOG_MSG("E", "Error getting state of $site_name\n", 14, $site_name);
        }

	return $retVal;
}

sub get_star_state() {
	my ($self) = @_;
	my ($found, $retVal, $q, @query, @star_state);

	$found = 0;
	$retVal = 1;
	@query = split('\n',$self->{'query'});

	foreach $q (@query) {
		if (!($q =~ /System State/) && $found == 0) {
			next;	
		}
		$found = 1;
		$retVal = 0;
		if ($q =~ /STAR/) {
			@star_state = split('\s+', $q);
			$self->{STATEINFO}{"STAR"} = $star_state[3];
			last;
		}
		if ($q =~ /Last Action Performed/) {
			last;
		}
	}

	if ($retVal) {
		VCSAG_LOG_MSG("E", "Error getting system state\n", 15);
	}

	return $retVal;
}

sub migrate() {
	my ($self) = @_;
	my ($retVal, $firstSite, $secondSite, $localSite, $ret, $trip);
	
	$retVal = 1;
	$firstSite = $self->{'firstSite'};
	$secondSite =  $self->{'secondSite'};
	$localSite =  $self->{'localSite'};
	$trip = 1;

	if (($self->{STATEINFO}{$firstSite} eq "Halted" && $self->{STATEINFO}{$secondSite} eq "Halted")) {
		$ret = $self->switch_to_local_site();
		if ($ret) {
			return $retVal;
		}
	} elsif ($self->{STATEINFO}{"STAR"} eq "Tripped") {
		if (!($self->{'tripTakeover'})) {
			VCSAG_LOG_MSG("E", "Cannot switch as TripTakeover is set to $self->{'tripTakeover'}. Administrator intervention required.\n", 17, $self->{'tripTakeover'});	
			return $retVal;
		}
		VCSAG_LOG_MSG("I", "Switching to $localSite site since TripTakeover is set to 1.\n", 18, $localSite);

		if (!($self->{STATEINFO}{$firstSite} eq "PathFail" && $self->{STATEINFO}{$secondSite} eq "PathFail")) {
			$ret = $self->disconnect($firstSite, $trip);
			if ($ret) {
				return $retVal;
			}
			$ret = $self->disconnect($secondSite, $trip);
			if ($ret) {
				return $retVal;
			}
		}
		$ret = $self->switch_to_local_site();
		if ($ret) {
			return $retVal;
		}
	} elsif ($self->{STATEINFO}{"STAR"} eq "Protected") {
		$ret = $self->disconnect($firstSite, $trip);
		if ($ret) {
			return $retVal;
		}
		$ret = $self->disconnect($secondSite, $trip);	
		if ($ret) {
			return $retVal;
		}
		$ret = $self->switch_to_local_site();
		if ($ret) {
			return $retVal;
		}
	} else {
		VCSAG_LOG_MSG("E", "Star is not in Protected/Tripped/Halted state. Cannot switch.\n", 19);
		return $retVal;
	}	

	$ret = $self->get_star_info();
	if ($ret) { 
		return $retVal;
	}

	$retVal = 0;
	return $retVal;
}

sub bring_WL_site_up() {
	my ($self) = @_;
	my ($enable, $syncSite, $asyncSite, $localSite, $firstSite, $secondSite, $otherSite);
	my ($ret, $retVal, $retSync, $retAsync, $otherSite, $trip);

	$retVal = 1;	
	$enable = 1;
	$retSync = 0;
	$retAsync = 0;
	$trip = 0;

	$localSite = $self->{'localSite'};
	$firstSite = $self->{'firstSite'};
	$secondSite = $self->{'secondSite'};
        $asyncSite = $self->{'asyncSite'};
        $syncSite = $self->{'syncSite'};

	if (($self->{STATEINFO}{$firstSite} eq "Halted") && 
		($self->{STATEINFO}{$secondSite} eq "Halted")) {
		$ret = $self->halt_reset();
		if ($ret) {
			return $retVal;
		}
        } elsif (($self->{STATEINFO}{$firstSite} eq "HaltFail") || 
		($self->{STATEINFO}{$secondSite} eq "HaltFail")) {
		$ret = $self->halt_reset_force();
		if ($ret) {
			return $retVal;
		}	
		if ($syncSite){
			$retSync = $self->disconnect($syncSite, $trip);
			if ($retSync) {
				VCSAG_LOG_MSG("E", "Failure disconnecting site $syncSite. ForceOnline is set to $self->{'forceOnline'}. \n", 85, $syncSite, $self->{'forceOnline'});
				if (!$self->{'forceOnline'}) {
					return $retVal;
				}
				$enable = 0;
			}
		}
		if ($asyncSite){
			$retAsync = $self->disconnect($asyncSite, $trip);
			if ($retAsync) {
				VCSAG_LOG_MSG("E", "Failure disconnecting site $asyncSite. ForceOnline is set to $self->{'forceOnline'}. \n", 85, $asyncSite, $self->{'forceOnline'});
				if (!$self->{'forceOnline'}) {
					return $retVal;
				}
				$enable = 0;
			}
		}
	} else {
		if (($self->{STATEINFO}{$firstSite} eq "PathFail") || 
			($self->{STATEINFO}{$firstSite} eq "PathFail;CleanReq")) {
			$ret = $self->cleanup($firstSite);
			$ret = $self->reset($firstSite);
		}

		if (($self->{STATEINFO}{$secondSite} eq "PathFail") || 
			($self->{STATEINFO}{$secondSite} eq "PathFail;CleanReq")) {
			$ret = $self->cleanup($secondSite);
			$ret = $self->reset($secondSite);
		}

		if ($self->{STATEINFO}{$firstSite} eq "Isolated") {
                        $ret = $self->disconnect($firstSite, $trip);
                }

		if ($self->{STATEINFO}{$secondSite} eq "Isolated") {
                        $ret = $self->disconnect($secondSite, $trip);
                }
	}

	if (($self->{'asyncTargetSite'} eq $localSite) && ($asyncSite)) {
		if ($self->{'syncTargetSite'} eq $asyncSite) {
			$otherSite = $self->{'workloadSite'};
		} else {
			$otherSite = $self->{'syncTargetSite'};
		}
		if (($self->{'keepDataOn'} eq "CONNECT") && ($self->{'keepDataFrom'} ne $localSite)) {
			if ($self->{'keepDataFrom'} eq $otherSite) {
				$ret = $self->connect($otherSite, $otherSite);
				if(!$retAsync) {
					$retAsync = $self->connect($asyncSite);
				}
			} elsif ($self->{'keepDataFrom'} eq $asyncSite) {
				if(!$retAsync) {
					$retAsync = $self->connect($asyncSite, $asyncSite);
				}
				$ret = $self->connect($otherSite);
			}
		} else {
			if(!$retAsync) {
				$retAsync = $self->connect($asyncSite);
			}
			$ret = $self->connect($otherSite);
		}

		if ($ret) {
			VCSAG_LOG_MSG("E", "Failure connecting to site $otherSite. ForceOnline is set to $self->{'forceOnline'}. \n", 86, $otherSite, $self->{'forceOnline'});
		}

		if ($retAsync) {
			VCSAG_LOG_MSG("E", "Failure connecting to site $asyncSite. ForceOnline is set to $self->{'forceOnline'}. \n", 86, $asyncSite, $self->{'forceOnline'});
			if (!$self->{'forceOnline'}) {
				return $retVal;
			}
		} else {
			$retAsync = $self->protect($asyncSite);
			if ($retAsync) {
				VCSAG_LOG_MSG("E", "Failure protecting site $asyncSite. ForceOnline is set to $self->{'forceOnline'}. \n", 64, $asyncSite, $self->{'forceOnline'});
				if (!$self->{'forceOnline'}) {
					return $retVal;
				}
			}
		}
	} else {
		if (!$syncSite && !$asyncSite) {
			$enable = 0;
			VCSAG_LOG_MSG("W", "Both SyncSite and AsyncSite attributes are not specified for resource $self->{'resname'}\n", 87, $self->{'resname'});
			if ($self->{'forceOnline'}) {
				VCSAG_LOG_MSG("I", "Resource $self->{'resname'} is going ONLINE since ForceOnline is set to $self->{'forceOnline'}\n", 65, $self->{'resname'}, $self->{'forceOnline'});
				$retVal = 0;
				return $retVal;
			} else {
				VCSAG_LOG_MSG("E", "Resource $self->{'resname'} is not going ONLINE since ForceOnline is set to $self->{'forceOnline'}\n", 88, $self->{'resname'}, $self->{'forceOnline'});
				return $retVal;
			}
		} elsif (($syncSite && !$asyncSite) && (!$retSync)) {
			$enable = 0; 
			if (($self->{'keepDataOn'} eq "CONNECT") && ($self->{'keepDataFrom'} eq $syncSite)) {
				$retSync = $self->connect($syncSite, $syncSite);
			} else {
				$retSync = $self->connect($syncSite);
			}
		} elsif ((!$syncSite && $asyncSite) && (!$retAsync)) {
			$enable = 0;
			if (($self->{'keepDataOn'} eq "CONNECT") && ($self->{'keepDataFrom'} eq $asyncSite)) {
				$retSync = $self->connect($asyncSite, $asyncSite);
			} else {
				$retSync = $self->connect($asyncSite);
			}
		} elsif (($syncSite && $asyncSite) && ((!$retSync) && (!$retAsync))) {
			if (($self->{'keepDataOn'} eq "CONNECT") && ($self->{'keepDataFrom'} ne $localSite)) {
				if ($self->{'keepDataFrom'} eq $syncSite) {
					$retSync = $self->connect($syncSite, $syncSite);
					$retAsync = $self->connect($asyncSite);
				} elsif ($self->{'keepDataFrom'} eq $asyncSite) {
					$retAsync = $self->connect($asyncSite, $asyncSite);
					$retSync = $self->connect($syncSite);
				}
			} else {
				$retSync = $self->connect($syncSite);
				$retAsync = $self->connect($asyncSite);
			}
		}

		if ($syncSite) {
			if ($retSync) {
				VCSAG_LOG_MSG("E", "Failure connecting to site $syncSite. ForceOnline is set to $self->{'forceOnline'}. \n", 86, $syncSite, $self->{'forceOnline'});
				if (!$self->{'forceOnline'}) {	
					return $retVal;
				}
				$enable = 0;
			} else {
				$retSync = $self->protect($syncSite);
				if ($retSync) {
					VCSAG_LOG_MSG("E", "Failure protecting site $syncSite. ForceOnline is set to $self->{'forceOnline'}. \n", 64, $syncSite, $self->{'forceOnline'});
					if (!$self->{'forceOnline'}) {
						return $retVal;
					}
					$enable = 0;
				}
			}
		}

		if ($asyncSite) {
			if ($retAsync) {
				VCSAG_LOG_MSG("E", "Failure connecting to site $asyncSite. ForceOnline is set to $self->{'forceOnline'}. \n", 86, $asyncSite, $self->{'forceOnline'});
				if (!$self->{'forceOnline'}) {
					return $retVal;
				}
				$enable = 0;
			} else {
				$retAsync = $self->protect($asyncSite);
				if ($retAsync) {
					VCSAG_LOG_MSG("E", "Failure protecting site $asyncSite. ForceOnline is set to $self->{'forceOnline'}. \n", 64, $asyncSite, $self->{'forceOnline'});
					if (!$self->{'forceOnline'}) {
						return $retVal;
					}
					$enable = 0;			
				}	
			}
		}

		if ($enable) {
			$ret = $self->enable();
			if ($ret) {
				VCSAG_LOG_MSG("W", "Star is not Enabled yet. Switching to remote site will be possible only when STAR is enabled.\n", 29);
				return $retVal;
			}
			$self->{'starEnabled'} = 1;			 
		}
	}

	$retVal = 0;
	return $retVal;
}

sub switch_to_local_site() {
	my ($self) = @_;
	my ($retVal, @output, $q, @query, $ret) ;

	$retVal = 1;
	@query = split('\n',$self->{'query'});

	if ($self->{'useMostRecentData'}) {
		$ret = $self->switch($self->{'localSite'}, $self->{'mostRecentSite'});
		if ($ret) {
			return $retVal;
		}
	} elsif (($self->{'keepDataOn'} eq "SWITCH") && ($self->{'keepDataFrom'})) {
		$ret = $self->switch($self->{'localSite'}, $self->{'keepDataFrom'});
	} else {
		$ret = $self->switch($self->{'localSite'}, $self->{'localSite'});
	}

	if ($ret) {
		return $retVal;
	}

	$retVal = 0;
	return $retVal;
}

sub switch() {
	my ($self, $site_name, $keep_data) = @_;
	my ($cmd, $retVal, $ret);

        $retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
	$cmd = $cmd . " -cg $self->{'name'} switch -site $site_name -keep_data $keep_data -nop" ;

	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18)  {
		VCSAG_LOG_MSG("E", "Error switching to $site_name site.\n", 20, $site_name);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub cleanup() {
	my ($self, $site_name) = @_;
	my ($cmd, $retVal, $ret); 

	$retVal = 1;	
	$cmd = $self->{'home'} . $SYMSTAR;
	$cmd = $cmd . " -cg $self->{'name'} cleanup -site $site_name -nop";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error cleaning up site $site_name site.\n", 23, $site_name);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}	

	return $retVal;
}

sub connect() {
	my ($self, $site_name, $keep_data) = @_;
	my ($cmd, $retVal, $ret);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
	if ($keep_data) {
		$cmd = $cmd . " -cg $self->{'name'} connect -site $site_name -keep_data $keep_data -nop";
	} else {
		$cmd = $cmd . " -cg $self->{'name'} connect -site $site_name -nop";
	}	
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
                VCSAG_LOG_MSG("E", "Error connecting to $site_name site.\n", 24, $site_name);
		$retVal = $ret;
        } else {
		$retVal = 0;
	}

        return $retVal;
}

sub disconnect() {
        my ($self, $site_name, $trip) = @_;
        my ($cmd, $retVal, $ret);

	$retVal = 1;
        $cmd = $self->{'home'} . $SYMSTAR;
	if($trip) {
		if (!($self->{STATEINFO}{$site_name} eq "Protected" || $self->{STATEINFO}{$site_name} eq "PathFailInProg")) {
			$retVal = 0;
			return $retVal;
		}
		$cmd = $cmd . " -cg $self->{'name'} disconnect -site $site_name -trip -nop ";
	} else {
		$cmd = $cmd . " -cg $self->{'name'} disconnect -site $site_name -nop ";	
	}	
	
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error disconnecting $site_name site.\n", 25, $site_name);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub protect() {
	my ($self, $site_name) = @_;
	my ($cmd, $retVal, $ret);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
	$cmd = $cmd . " -cg $self->{'name'} protect -site $site_name -nop ";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error protecting $site_name site.\n", 26, $site_name);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub unprotect() {
        my ($self, $site_name) = @_;
        my ($cmd, $retVal, $ret);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
        $cmd = $cmd . " -cg $self->{'name'} unprotect -site $site_name -nop ";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error unprotecting $site_name site.\n", 27, $site_name);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub enable() {
	my ($self) = @_;
	my ($cmd, $retVal, $ret);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
	if ($self->{'onlineOnState'} eq "STAR_PROTECTED"){
		$cmd = $cmd . " -cg $self->{'name'} enable -nop ";
	} else {
		VCSAG_LOG_MSG("W", "Star is not Enabled yet. Switching to remote site will be possible only when STAR is enabled.\n", 29);
		$cmd = $cmd . " -cg $self->{'name'} enable -nop &";
		VCSAG_LOG_MSG("N", "issuing command: $cmd", 101, $cmd);			
		$ret = system($cmd);
		if ($ret) {
			VCSAG_LOG_MSG("E", "Command failed:\treturn code $ret\n\n", 30, $ret);
		} else {
			VCSAG_LOG_MSG("N", "command succeeded with return code $ret", 102, $ret);
			$retVal = 0;
		}
		return $retVal;
	}
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Failure enabling STAR. ForceOnline is set to $self->{'forceOnline'}.\n", 66, $self->{'forceOnline'});
		$retVal = $ret;
		if ($self->{'forceOnline'}) {
			VCSAG_LOG_MSG("E", "Resource $self->{'resname'} is going ONLINE since ForceOnline is set to $self->{'forceOnline'}\n", 65, $self->{'resname'}, $self->{'forceOnline'});
			$retVal = 0;		
		}
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub enable_action() {
	my ($self) = @_;
	my ($cmd, $retVal, $ret);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
	$cmd = $cmd . " -cg $self->{'name'} enable -nop ";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error enabling STAR.\n", 68);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub disable() {
        my ($self) = @_;
        my ($cmd, $retVal, $ret);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
        $cmd = $cmd . " -cg $self->{'name'} disable -nop ";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error disabling STAR.\n", 32);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub isolate() {
	my ($self, $site_name) = @_;
        my ($cmd, $retVal, $ret);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
        $cmd = $cmd . " -cg $self->{'name'} isolate -site $site_name -nop ";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
                VCSAG_LOG_MSG("E", "Error isolating $site_name\n", 33, $site_name);
		$retVal = $ret;
        } else {
		$retVal = 0;
	}

	return $retVal;
}

sub halt() {
	my ($self) = @_;
	my ($cmd, $retVal, $ret);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
	$cmd = $cmd . " -cg $self->{'name'} halt -nop ";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error doing halt\n", 34);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub halt_reset() {
        my ($self) = @_;
        my ($cmd, $retVal, $ret);

	$retVal = 1;
        $cmd = $self->{'home'} . $SYMSTAR;
        $cmd = $cmd . " -cg $self->{'name'} halt -reset -nop ";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error doing halt reset\n", 35);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub halt_reset_force() {
        my ($self) = @_;
        my ($cmd, $retVal, $ret);

	$retVal = 1;
        $cmd = $self->{'home'} . $SYMSTAR;
        $cmd = $cmd . " -cg $self->{'name'} halt -reset -force -nop ";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error doing halt reset force\n", 36);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub reset() {
	my ($self, $site_name) = @_;
	my ($cmd, $retVal, $ret);

	$retVal = 1;
	$cmd = $self->{'home'} . $SYMSTAR;
	$cmd = $cmd . " -cg $self->{'name'} reset -site $site_name -nop ";
	push(@CMDS, $cmd);
	$ret = run_cmds(@CMDS);
	if ($ret && $ret != 18) {
		VCSAG_LOG_MSG("E", "Error resetting site $site_name\n", 37, $site_name);
		$retVal = $ret;
	} else {
		$retVal = 0;
	}

        return $retVal;
}

sub _do {
	my ($do, $line, $ret, $retVal);

        ($do) = (@_);
        VCSAG_LOG_MSG("N", "issuing command: $do", 101, "$do");
        open(INPUT,"$do 2>&1 |") || die("command $do failed");
        while ($line = <INPUT>) { chomp($line); $ret = ($ret) ? "$ret\n$line" : $line; }
        close INPUT;
        $_DOE=$?;
        $_DO = $ret;
	$retVal = $_DOE / 256;
        if ($retVal && $retVal != 18) {
		VCSAG_LOG_MSG("E", "Command failed:\n$_DO\n\treturn code $retVal\n\n", 89, $_DO, $retVal);
		$retVal = $ret;
        } else {
                VCSAG_LOG_MSG("N", "command succeeded with return code $retVal", 102, $retVal);
		$retVal = 0;
        }

        return $retVal;
}

sub run_cmds {
	my ($cmd, $retVal);

	$retVal = 0;

	foreach $cmd (@CMDS) {
		$retVal = _do($cmd);
		last; 
	}
	@CMDS = ();

	return $retVal;
}

sub get_options_file_data() {
	my ($self) = @_;
	my ($retVal, $cmd, $found, $result, @output, $l, @lines);

	$retVal = 1;
	$found = 0;

	$cmd = $self->{'home'} . $SYMSTAR;
	$cmd = $cmd . " -cg $self->{'name'} show";
	VCSAG_LOG_MSG("N", "issuing command: $cmd", 101, $cmd);
	$result = `$cmd`;
	if (!$result) {
		VCSAG_LOG_MSG("E", "Command failed:\n $cmd \n", 62, $cmd);
		return $retVal;
	}
	@lines = split('\n', $result);
	foreach $l (@lines) {
                if ($l =~ /WorkloadSite/) {
			@output= split('\s+', $l);
			$self->{'workloadSite'} = $output[3];
			$found = $found + 1;
		}elsif ($l =~ /SyncTargetSite/) {
			@output= split('\s+', $l);
			$self->{'syncTargetSite'} = $output[3];
			$found = $found + 1;
		}elsif ($l =~ /AsyncTargetSite/) {
			@output= split('\s+', $l);
			$self->{'asyncTargetSite'} = $output[3];
			$found = $found + 1;
			last;
		}
	}

	if (!$self->{'workloadSite'} || ! $self->{'syncTargetSite'} || ! $self->{'asyncTargetSite'}) {
		VCSAG_LOG_MSG("E", "Error getting options file data\n", 72);
	} else {
		$retVal = 0;
	}

	return $retVal;
}

sub extend_action_timeout() {
	my ($self) = @_;
	my ($retVal, $tmoExtensionFile);

	$retVal = 1;
	$tmoExtensionFile = "/var/VRTSvcs/log/tmp/" . "$self->{'resname'}" . ".tmo";
	open TMOEXTENSIONFILE , ">$tmoExtensionFile" || die ("Cannot open $tmoExtensionFile file!\n");
	print TMOEXTENSIONFILE "$self->{'actionTimeout'}";
	close TMOEXTENSIONFILE;
	$retVal = 0;
	return $retVal; 	
}

sub validate_sitename() {
	my ($self, $siteName) = @_;
	my ($retVal, $ret);
	
	$retVal = 1;

	if (($self->{'workloadSite'} ne $siteName) &&  ($self->{'syncTargetSite'} ne $siteName) && 
		($self->{'asyncTargetSite'} ne $siteName)) {
		VCSAG_LOG_MSG("E", "Site $siteName does not belong to this Star configuration\n", 96, $siteName);
	} else {
		$retVal = 0;
	}

        return $retVal;
}	

sub get_lockfilename {
	my ($self) = shift;
	my $typ;
	my $lock_file;
	my $lock_dir = ""; 

	if (!defined($self->{'resource_type'}) || 
		(length($self->{'resource_type'}) == 0)) {

		$typ = `$ag_i18n_inc::HARES -value $self->{'resname'} Type`;
		chomp $typ;
	} else {
		$typ = $self->{'resource_type'};
	}

	if ($^O =~ /MSWin32/i) {
		$lock_dir = $ag_i18n_inc::VCSLOG . "\\lock\\" . $typ . "\\";
	} else {
		$lock_dir = $ag_i18n_inc::VCSLOG . "/lock/" . $typ . "/";
	}

	$lock_file = $lock_dir . ".VCS_" . $typ . "_" . $self->{'resname'} . 
		"_Online_Lockfile.DO-NOT-REMOVE";

	return $lock_file;
}

sub create_lock_file {
	my ($self) = shift;
	my ($lock_file) = $self->get_lockfilename();

	my (undef, $lock_dir, undef) = fileparse($lock_file);

	if (! -d $lock_dir) {
		mkdir  $lock_dir or 
			VCSAG_LOG_MSG("E", "Could not create lock directory $lock_dir",
				1001, "$lock_dir");
	}

	open(lfh, ">$lock_file") or 
		VCSAG_LOG_MSG("E", "Could not create file: $lock_file", 1002,
			 "$lock_file");

	close(lfh);
}

sub remove_lock_file {
	my($self) = shift;
	my ($lock_file) = $self->get_lockfilename();

	if (-f "$lock_file") {
		unlink("$lock_file") or 
			VCSAG_LOG_MSG("E", "Could not delete file: $lock_file", 1003,
				 "$lock_file");
	}
}

sub exists_lock_file {
	my ($self) = shift;
	my $retval = 0;
	my ($lock_file) = $self->get_lockfilename();

	if (-f "$lock_file") {
		$retval = 1;
	}
	return $retval;
}

sub is_parent_online {
	my ($self) = shift;
	my ($p_res, $ret, $tmpstr, $sys);
	my @p_res_list = ();
	my $is_online = 0;

	$tmpstr = `$ag_i18n_inc::HASYS -nodeid`;
	chomp $tmpstr;
	$tmpstr =~ s/^.*Node name:[\s+](.*)[\s+]Node ID:.*$/$1/ ;
	$self->{'nodename'} = $sys = $tmpstr;

	# Are there any resources that depend on this resource
	@p_res_list = $self->get_res_deps();

	if ($#p_res_list >= 0) { # Parent resources were found
		foreach $p_res (@p_res_list) {

			$ret = system("$ag_i18n_inc::HARES -wait $p_res Probed 1 "
					." -sys $sys -time 120 2>/dev/null");
			if ($ret == 0) {
				my ($st) = split('\n', `$ag_i18n_inc::HARES -state $p_res -sys $sys`);		
				chomp $st;
				if ($st =~ /ONLINE/) {
					$is_online = 1;
					last;
				}
			}
		}
	} else { # No parent resources
		if ($self->is_parent_sg_res_online()) {
			$is_online = 1;	
		}
	}

	return $is_online;
}

#
# Return 1 if at least one OnOff/OnOnly resource from a parent service
# group is online.
#

sub is_parent_sg_res_online {
	my ($self) = shift;
	my ($sg_name, $p_sg_name, $line, $ret);
	my @lines = ();
	my @fields = ();
	my @valid_dep_lines = ();
	my $is_online = 0;
	my $sys = $self->{'nodename'};
	
	#
	# Get parent service groups that have an online local/online global
	# dependency to our service group
	#

	$sg_name = `$ag_i18n_inc::HARES -value $self->{'resname'} Group`;
	chomp $sg_name;

	(@lines) = split('\n', `$ag_i18n_inc::HAGRP -dep $sg_name`);
	chomp @lines;

	# VCS specific code --->
	(@valid_dep_lines) = grep (/online[\s+](local|global)/, @lines );

	foreach $line (@valid_dep_lines) {
		(@fields) = split('\s+', $line);

		if ($fields[1] eq $sg_name) {
			$p_sg_name = $fields[0];
		} else {
			next;
		}

		#
		# If at least one OnOff or OnOnly probed resource in the ParentSG
		# has gone online, then the SRDF resource can be online as well
		#

		$ret = system("$ag_i18n_inc::HAGRP -wait $p_sg_name Probed 1 -sys $sys"
				 . " -time 120 2>/dev/null");

		if ($ret == 0) {
			my @p_on_res_list = $self->get_res_list($p_sg_name, "ONLINE");	

			#
			# At least 1 online resource was found. Note: $#X == 0
			# implies @X has 1 element
			#
			if ($#p_on_res_list >= 0){

				#
				# We found at least one ONLINE resource. If it is of type 
				# OnOff or OnOnly, we can continue to remain ONLINE
				#   
				foreach my $p_res (@p_on_res_list) {
					my $p_type = `$ag_i18n_inc::HARES -value $p_res Type`;
					chomp $p_type;
					my $p_oprn_type = `$ag_i18n_inc::HATYPE -value $p_type Operations`;
					chomp $p_oprn_type;
					if ($p_oprn_type =~ /None/i ) {
						next;
					}

					$is_online = 1;
					last;
				}
			}
		}
		last if ($is_online == 1);
	}

	return $is_online;
}

#
# Returns the names of resources that belong to a given group and
# are in particular state
#
sub get_res_list {
	my ($self) = shift;
	my ($sg_name, $res_state) = @_;
	my @retarr = ();
	my @tmparr = ();
	my ($ret, $nodename, $resname);

	$ret = run_cmd("$ag_i18n_inc::HARES -list Group=$sg_name State=$res_state", 
				\@tmparr, 1);

	if ($ret) {
		foreach (@tmparr) {
			/^#/ && next;
			($resname, $nodename)  = split;

			if ((!grep(/^${resname}$/, @retarr)) &&
				($nodename =~ /$self->{'nodename'}/)) {

				push (@retarr, $resname);
			}
		}	
	}
	return @retarr;	
}


#
# Returns the names of parent resources
#
sub get_res_deps {
	my ($self) = shift;

	my ($sg_name, $res_state) = @_;
	my @retarr = ();
	my @tmparr = ();
	my ($ret, $grpname, $p_resname, $c_resname);

	$ret = run_cmd("$ag_i18n_inc::HARES -dep $self->{'resname'}", \@tmparr, 1);

	if ($ret) {
		foreach (@tmparr) {
			/^#/ && next;
			($grpname, $p_resname, $c_resname)  = split;
		
			#
			# Note: with VCSOne, the 'resname' would be <grpname>.<resname>
			# whereas the $c_resname would only have the trailing <resname> 
			# part
			#
		
			if ((!grep(/^${p_resname}$/, @retarr)) && 
				( $c_resname =~ /$self->{'resname'}$/ )) {
				push (@retarr, $p_resname);
			}
		}	
	}
	
	return @retarr;	
}


sub run_cmd {
	my ($cmd, $output_ref, $ignore_failure) = @_;
	my $retval = 1;

	@{$output_ref} = `$cmd 2>&1`;

	if ($? != 0) {
		$retval = 0;
		unless (1 == $ignore_failure ) {
			VCSAG_LOG_MSG("W", "Command: \'$cmd\' failed !", 1007,
					"$cmd");
		}
	} else {
		chomp @{$output_ref};
	}

	return $retval;
}



1;
